#ifndef UTILS_H
#define UTILS_H

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include <GL/gl.h>
#include <GL/glu.h>
#include <SDL/SDL.h>
#include <SDL/SDL_ttf.h>
#include <sys/types.h>
#include <dirent.h>

#define titre_fenetre   "Emulation Visualisation 27C256 DIGIFANT"
#define fontpath        "fichiers/fonts/bitmap4.ttf"
#define screenwidth     1024
#define screenheight    768
#define dt              10

int nextpoweroftwo(int x);
void init_sdl();
void SDL_GL_RenderText(char *text,TTF_Font *font,SDL_Color color,SDL_Rect *location);
void glEnable2D();
void glDisable2D();
void init_gl();
float Hue_2_RGB(float v1,float v2,float vH);
void HSL2RGB(float h, float s,float l,unsigned char * r,unsigned char * g,unsigned char *b);

#endif
